@@ -10,7 +10,7 @@ import logging
 import time
 from dataclasses import dataclass
 from secrets import token_hex
-from typing import Any, Optional, Union
+from typing import Any, Generator, Optional, Union
 
 import aiohttp
 from aiohttp.http_websocket import json
@@ -31,92 +31,135 @@ def pretty_modulename(e: Exception) -> str:
     return modulename
 
 
-async def ws_run_forever(
-    url: StrOrURL,
-    session: aiohttp.ClientSession,
-    event: asyncio.Event,
-    *,
-    send_str: Optional[Union[str, list[str]]] = None,
-    send_bytes: Optional[Union[bytes, list[bytes]]] = None,
-    send_json: Any = None,
-    hdlr_str=None,
-    hdlr_bytes=None,
-    hdlr_json=None,
-    auth=_Auth,
-    **kwargs: Any,
-) -> None:
-    if all([hdlr_str is None, hdlr_json is None]):
-        hdlr_json = pybotters.print_handler
-    iscorofunc_str = asyncio.iscoroutinefunction(hdlr_str)
-    iscorofunc_bytes = asyncio.iscoroutinefunction(hdlr_bytes)
-    iscorofunc_json = asyncio.iscoroutinefunction(hdlr_json)
-    while not session.closed:
-        cooldown = asyncio.create_task(asyncio.sleep(60.0))
-        try:
-            async with session.ws_connect(url, auth=auth, **kwargs) as ws:
-                event.set()
-                if '_authtask' in ws.__dict__:
-                    await ws.__dict__['_authtask']
-                if send_str is not None:
-                    if isinstance(send_str, list):
-                        await asyncio.gather(*[ws.send_str(item) for item in send_str])
-                    else:
-                        await ws.send_str(send_str)
-                if send_bytes is not None:
-                    if isinstance(send_bytes, list):
-                        await asyncio.gather(
-                            *[ws.send_bytes(item) for item in send_bytes]
-                        )
-                    else:
-                        await ws.send_bytes(send_bytes)
-                if send_json is not None:
-                    if isinstance(send_json, list):
-                        await asyncio.gather(
-                            *[ws.send_json(item) for item in send_json]
-                        )
-                    else:
-                        await ws.send_json(send_json)
-                async for msg in ws:
-                    if msg.type == aiohttp.WSMsgType.TEXT:
-                        if hdlr_str is not None:
-                            try:
-                                if iscorofunc_str:
-                                    await hdlr_str(msg.data, ws)
-                                else:
-                                    hdlr_str(msg.data, ws)
-                            except Exception as e:
-                                logger.exception(f'{pretty_modulename(e)}: {e}')
-                        if hdlr_json is not None:
-                            try:
-                                data = msg.json()
-                            except json.decoder.JSONDecodeError:
-                                pass
-                            else:
+class WebSocketRunner:
+    def __init__(
+        self,
+        url: StrOrURL,
+        session: aiohttp.ClientSession,
+        *,
+        send_str: Optional[Union[str, list[str]]] = None,
+        send_bytes: Optional[Union[bytes, list[bytes]]] = None,
+        send_json: Any = None,
+        hdlr_str=None,
+        hdlr_bytes=None,
+        hdlr_json=None,
+        auth=_Auth,
+        **kwargs: Any,
+    ) -> None:
+        self.conneted = False
+        self._event = asyncio.Event()
+        self._task = asyncio.create_task(
+            self._run_forever(
+                url,
+                session,
+                send_str=send_str,
+                send_bytes=send_bytes,
+                send_json=send_json,
+                hdlr_str=hdlr_str,
+                hdlr_bytes=hdlr_bytes,
+                hdlr_json=hdlr_json,
+                auth=auth,
+                **kwargs,
+            )
+        )
+
+    async def _run_forever(
+        self,
+        url: StrOrURL,
+        session: aiohttp.ClientSession,
+        *,
+        send_str: Optional[Union[str, list[str]]] = None,
+        send_bytes: Optional[Union[bytes, list[bytes]]] = None,
+        send_json: Any = None,
+        hdlr_str=None,
+        hdlr_bytes=None,
+        hdlr_json=None,
+        auth=_Auth,
+        **kwargs: Any,
+    ) -> None:
+        if all([hdlr_str is None, hdlr_json is None]):
+            hdlr_json = pybotters.print_handler
+        iscorofunc_str = asyncio.iscoroutinefunction(hdlr_str)
+        iscorofunc_bytes = asyncio.iscoroutinefunction(hdlr_bytes)
+        iscorofunc_json = asyncio.iscoroutinefunction(hdlr_json)
+        while not session.closed:
+            cooldown = asyncio.create_task(asyncio.sleep(60.0))
+            try:
+                async with session.ws_connect(url, auth=auth, **kwargs) as ws:
+                    self.conneted = True
+                    self._event.set()
+                    if '_authtask' in ws.__dict__:
+                        await ws.__dict__['_authtask']
+                    if send_str is not None:
+                        if isinstance(send_str, list):
+                            await asyncio.gather(
+                                *[ws.send_str(item) for item in send_str]
+                            )
+                        else:
+                            await ws.send_str(send_str)
+                    if send_bytes is not None:
+                        if isinstance(send_bytes, list):
+                            await asyncio.gather(
+                                *[ws.send_bytes(item) for item in send_bytes]
+                            )
+                        else:
+                            await ws.send_bytes(send_bytes)
+                    if send_json is not None:
+                        if isinstance(send_json, list):
+                            await asyncio.gather(
+                                *[ws.send_json(item) for item in send_json]
+                            )
+                        else:
+                            await ws.send_json(send_json)
+                    async for msg in ws:
+                        if msg.type == aiohttp.WSMsgType.TEXT:
+                            if hdlr_str is not None:
                                 try:
-                                    if iscorofunc_json:
-                                        await hdlr_json(data, ws)
+                                    if iscorofunc_str:
+                                        await hdlr_str(msg.data, ws)
                                     else:
-                                        hdlr_json(data, ws)
+                                        hdlr_str(msg.data, ws)
                                 except Exception as e:
                                     logger.exception(f'{pretty_modulename(e)}: {e}')
-                    elif msg.type == aiohttp.WSMsgType.BINARY:
-                        if hdlr_bytes is not None:
-                            try:
-                                if iscorofunc_bytes:
-                                    await hdlr_bytes(msg.data, ws)
+                            if hdlr_json is not None:
+                                try:
+                                    data = msg.json()
+                                except json.decoder.JSONDecodeError:
+                                    pass
                                 else:
-                                    hdlr_bytes(msg.data, ws)
-                            except Exception as e:
-                                logger.exception(f'{pretty_modulename(e)}: {e}')
-                    elif msg.type == aiohttp.WSMsgType.ERROR:
-                        break
-        except (
-            aiohttp.WSServerHandshakeError,
-            aiohttp.ClientOSError,
-            ConnectionResetError,
-        ) as e:
-            logger.warning(f'{pretty_modulename(e)}: {e}')
-        await cooldown
+                                    try:
+                                        if iscorofunc_json:
+                                            await hdlr_json(data, ws)
+                                        else:
+                                            hdlr_json(data, ws)
+                                    except Exception as e:
+                                        logger.exception(f'{pretty_modulename(e)}: {e}')
+                        elif msg.type == aiohttp.WSMsgType.BINARY:
+                            if hdlr_bytes is not None:
+                                try:
+                                    if iscorofunc_bytes:
+                                        await hdlr_bytes(msg.data, ws)
+                                    else:
+                                        hdlr_bytes(msg.data, ws)
+                                except Exception as e:
+                                    logger.exception(f'{pretty_modulename(e)}: {e}')
+                        elif msg.type == aiohttp.WSMsgType.ERROR:
+                            break
+            except (
+                aiohttp.WSServerHandshakeError,
+                aiohttp.ClientOSError,
+                ConnectionResetError,
+            ) as e:
+                logger.warning(f'{pretty_modulename(e)}: {e}')
+            self.conneted = False
+            self._event.clear()
+            await cooldown
+
+    async def wait(self) -> None:
+        await self._event.wait()
+
+    def __await__(self) -> Generator[Any, None, None]:
+        return self._task.__await__()
 
 
 class Heartbeat:
@@ -156,6 +199,20 @@ class Heartbeat:
             await ws.send_str('{"method":"server.ping","params":[],"id":123}')
             await asyncio.sleep(10.0)
 
+    @staticmethod
+    async def okx(ws: aiohttp.ClientWebSocketResponse):
+        while not ws.closed:
+            await ws.send_str('ping')
+            await asyncio.sleep(15.0)
+
+    @staticmethod
+    async def bitget(ws: aiohttp.ClientWebSocketResponse):
+        while not ws.closed:
+            await ws.send_str('ping')
+            # 公式サンプルが25秒ごとに送っていたので25秒に設定
+            # https://github.com/BitgetLimited/v3-bitget-api-sdk/blob/master/bitget-python-sdk-api/bitget/ws/bitget_ws_client.py
+            await asyncio.sleep(25.0)
+
 
 class Auth:
     @staticmethod
@@ -167,7 +224,7 @@ class Auth:
             AuthHosts.items[ws._response.url.host].name
         ][1]
 
-        timestamp = int(time.time())
+        timestamp = int(time.time() * 1000)
         nonce = token_hex(16)
         sign = hmac.new(
             secret, f'{timestamp}{nonce}'.encode(), digestmod=hashlib.sha256
@@ -187,6 +244,8 @@ class Auth:
         async for msg in ws:
             if msg.type == aiohttp.WSMsgType.TEXT:
                 data = msg.json()
+                if 'error' in data:
+                    logger.warning(data)
                 if 'id' in data:
                     if data['id'] == 'auth':
                         break
@@ -279,11 +338,90 @@ class Auth:
         async for msg in ws:
             if msg.type == aiohttp.WSMsgType.TEXT:
                 data = msg.json()
+                if 'error' in data:
+                    if data['error'] is not None:
+                        logger.warning(data)
                 if data['result'] == {'status': 'success'}:
                     break
             elif msg.type == aiohttp.WSMsgType.ERROR:
                 break
 
+    @staticmethod
+    async def okx(ws: aiohttp.ClientWebSocketResponse):
+        if not ws._response.url.path.endswith('private'):
+            return
+
+        key: str = ws._response._session.__dict__['_apis'][
+            AuthHosts.items[ws._response.url.host].name
+        ][0]
+        secret: bytes = ws._response._session.__dict__['_apis'][
+            AuthHosts.items[ws._response.url.host].name
+        ][1]
+        passphrase: bytes = ws._response._session.__dict__['_apis'][
+            AuthHosts.items[ws._response.url.host].name
+        ][2]
+
+        timestamp = str(int(time.time()))
+        text = f'{timestamp}GET/users/self/verify'
+        sign = base64.b64encode(
+            hmac.new(secret, text.encode(), digestmod=hashlib.sha256).digest()
+        ).decode()
+        msg = {
+            'op': 'login',
+            'args': [
+                {
+                    'apiKey': key,
+                    'passphrase': passphrase,
+                    'timestamp': timestamp,
+                    'sign': sign,
+                }
+            ],
+        }
+        await ws.send_json(msg)
+        async for msg in ws:
+            if msg.type == aiohttp.WSMsgType.TEXT:
+                try:
+                    data = msg.json()
+                    if data['event'] == 'error':
+                        logger.warning(data)
+                    if data['event'] == 'login':
+                        break
+                except json.JSONDecodeError:
+                    pass
+            elif msg.type == aiohttp.WSMsgType.ERROR:
+                break
+
+    @staticmethod
+    async def bitget(ws: aiohttp.ClientWebSocketResponse):
+        key: str = ws._response._session.__dict__['_apis'][
+            AuthHosts.items[ws._response.url.host].name
+        ][0][0]
+        secret: bytes = ws._response._session.__dict__['_apis'][
+            AuthHosts.items[ws._response.url.host].name
+        ][1]
+        passphrase: bytes = ws._response._session.__dict__['_apis'][
+            AuthHosts.items[ws._response.url.host].name
+        ][0][1]
+
+        timestamp = int(round(time.time()))
+        sign = base64.b64encode(
+            hmac.new(
+                secret, f'{timestamp}GET/user/verify'.encode(), digestmod=hashlib.sha256
+            ).digest()
+        ).decode()
+        msg = {
+            'op': 'login',
+            'args': [
+                {
+                    'api_key': key,
+                    'passphrase': passphrase,
+                    'timestamp': str(timestamp),
+                    'sign': sign,
+                }
+            ],
+        }
+        await ws.send_json(msg)
+
 
 @dataclass
 class Item:
@@ -309,6 +447,10 @@ class HeartbeatHosts:
         'testnetws.binanceops.com': Heartbeat.binance,
         'phemex.com': Heartbeat.phemex,
         'testnet.phemex.com': Heartbeat.phemex,
+        'ws.okx.com': Heartbeat.okx,
+        'wsaws.okx.com': Heartbeat.okx,
+        'wspap.okx.com': Heartbeat.okx,
+        'ws.bitget.com': Heartbeat.bitget,
     }
 
 
@@ -319,6 +461,10 @@ class AuthHosts:
         'ftx.com': Item('ftx', Auth.ftx),
         'phemex.com': Item('phemex', Auth.phemex),
         'testnet.phemex.com': Item('phemex_testnet', Auth.phemex),
+        'ws.okx.com': Item('okx', Auth.okx),
+        'wsaws.okx.com': Item('okx', Auth.okx),
+        'wspap.okx.com': Item('okx', Auth.okx),
+        'ws.bitget.com': Item('bitget', Auth.bitget),
     }
 
 
